/*
 * (C) Copyright 2000
 * Rob Taylor, Flying Pig Systems. robt@flyingpig.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc824x.h>
#include <pci.h>
#include "gluebert.h"

int checkboard (void)
{
	/*TODO: Check processor type */

	puts (	"Board: Bert "
#ifdef CONFIG_MPC8245
		"8241\n"
#endif
    );
	return 0;
}

long int initdram (int board_type)
{
	int		 i;
	volatile uchar * addr;

    /* Do a memory detection test */
    for (i = 1; i <= CFG_MAX_RAM_SIZE; i<<=1) {
        addr = (uchar *)(i-1);
        *addr = 0xA5;
    }

    for (i = 1; i <= CFG_MAX_RAM_SIZE; i<<=1) {
        addr = (uchar *)(i-1);
        if (*addr != 0xA5) break;
    }
	
    return (i>>1);
}

int board_pre_init(void)
{
    return 0;
}

/*
 * Initialize PCI Devices, report devices found.
 */
#ifndef CONFIG_PCI_PNP
static struct pci_config_table pci_bert_config_table[] = {
	{ PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, BERT_GEA0_DEV_NUM, PCI_ANY_ID,
	  pci_cfgfunc_config_device, 
          { 0,
			BERT_GEA0_PCI_BASE,
			PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER }
    },
	{ PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, BERT_GEA1_DEV_NUM, PCI_ANY_ID,
	  pci_cfgfunc_config_device, 
          { 0,
			BERT_GEA1_PCI_BASE,
			PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER }
    },
	{ PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, BERT_SF_DEV_NUM, PCI_ANY_ID,
	  pci_cfgfunc_config_device, 
          { 0,
			BERT_SF_PCI_BASE,
			PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER }
    },
	{ PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, BERT_MEZ_DEV_NUM, PCI_ANY_ID,
	  pci_cfgfunc_config_device, 
          { 0,
			BERT_MEZ_PCI_BASE,
			PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER }
    },
	{ PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, BERT_ERNIE_DEV_NUM, PCI_ANY_ID,
	  pci_cfgfunc_config_device, 
          { 0,
			BERT_ERNIE_PCI_BASE,
			PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER }
    },
	{ }
};
#endif

struct pci_controller hose = {
#ifndef CONFIG_PCI_PNP
	config_table: pci_bert_config_table,
#endif
};

void pci_init_board(void)
{
    uchar *pGluebert = NULL;
    uchar reset_val;

    uchar *eummbar = NULL;

    /* HACK - Enable Inbound Translation Registers - HACK */
    /* HACK - Should this be done somewhere else? */
    eummbar = (uchar *)(CFG_EUMB_ADDR + 0x02310);
    *(uint *)eummbar = 0x1e000000;
    
    eummbar = (uchar *)(CFG_EUMB_ADDR + 0x02330);
    *(uint *)eummbar = 0x1e000000;

    /* Initialize ERCRs for Gluebert Access */
    CONFIG_WRITE_WORD( ERCR1, 0x81042205);
    CONFIG_WRITE_WORD( ERCR2, 0x8d042205);
    
    /* Do some Gluebert work to enable PCI devices... */
    /* clear global reset in GlueBert */
    pGluebert = (uchar *)(RESET_BASE + GLOBAL_RESET_OFFSET);
    reset_val = *pGluebert;
    reset_val = reset_val & ~GLOBAL_GLBL_RST;
    *pGluebert = reset_val;

    /* Clear 5690s and 5671 reset in GlueBert */
    pGluebert = (uchar *)(RESET_BASE + GEA_RESET_OFFSET);
    reset_val = *pGluebert;
    reset_val = reset_val & ~(GEA_RESET_MASK);
    *pGluebert = reset_val;

    /* Clear PHY reset in GlueBert */
    pGluebert = (uchar *)(RESET_BASE + PHY_RESET_OFFSET);
    reset_val = *pGluebert;
    reset_val = reset_val & ~(PHY_RESET_MASK);
    *pGluebert = reset_val;

    /* Clear LED reset in GlueBert */
    pGluebert = (uchar *)(RESET_BASE + LED_DRIVER_RESET_OFFSET);
    reset_val = *pGluebert;
    reset_val = reset_val & ~(LED_DRIVER_RESET_MASK);
    *pGluebert = reset_val;

    /* Enable all the interrupts (at the Gluebert level) */
    pGluebert = (uchar *)(INTERRUPT_BASE + IRQ0_MASK_REG_OFFFSET);
    reset_val = *pGluebert;
    reset_val = reset_val & ~(IRQ0_MAC0_PCI_INT | IRQ0_MAC1_PCI_INT);
    *pGluebert = reset_val;
    
    pGluebert = (uchar *)(INTERRUPT_BASE + IRQ3_MASK_REG_OFFSET);
    reset_val = *pGluebert;
    reset_val = reset_val & ~(IRQ3_5671_INT);
    *pGluebert = reset_val;
    
    pGluebert = (uchar *)(RESET_BASE + PHY_RESET_OFFSET);
    reset_val = *pGluebert;
    reset_val = reset_val & ~(PHY_RESET_MASK);
    *pGluebert = reset_val;

    pGluebert = (uchar *)(RESET_BASE + LED_DRIVER_RESET_OFFSET);
    reset_val = *pGluebert;
    reset_val = reset_val & ~(LED_DRIVER_RESET_MASK);
    *pGluebert = reset_val;

	pci_mpc824x_init(&hose);
}
