/*
 * Memory sub-system initialization code for BCM94704CPCI.
 *
 * Copyright (c) 2003	LVL7 Systems, Inc.
 * John W. Linville <linville@lvl7.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>
#include <asm/regdef.h>

#include "bcm47xx.h"

#define	MEMCTRL_CONFIG_VAL	0x0006b010
#define	MEMCTRL_CONFIG_VAL_64MB	0x0006b008
#define	MEMCTRL_RDNCDLCOR_VAL	0x14141414
#define	MEMCTRL_MISCDLYCTL_VAL	0x20000000
#define	MEMCTRL_REFRESH_VAL	0x0000861a
#define	MEMCTRL_MODEREG_VAL	0x00000023

	.globl	memsetup
memsetup:

	/* Memory controller core reset */
	li	a0, 0xB8008f98
	li	a1, 0x00030001
	sw	a1, 0(a0)
	lw	a1, 0(a0)	/* use multiple loads to insert a delay */
	lw	a1, 0(a0)
	lw	a1, 0(a0)
	li	a1, 0x00030000
	sw	a1, 0(a0)
	lw	a1, 0(a0)	/* use multiple loads to insert a delay */
	lw	a1, 0(a0)
	lw	a1, 0(a0)
	li	a1, 0x00010000
	sw	a1, 0(a0)
	lw	a1, 0(a0)	/* use multiple loads to insert a delay */
	lw	a1, 0(a0)
	lw	a1, 0(a0)

	/* Memory controller init */
	li	a0, 0xB8008000	# SDRAM registers

	/* Program SDRAM configuration values */
	li	a1, MEMCTRL_CONFIG_VAL
	sw	a1, MEMCTRL_CONFIG(a0)

	/* Program Read NCDL Correction values */
	li	a1, MEMCTRL_RDNCDLCOR_VAL
	sw	a1, MEMCTRL_RDNCDLCOR(a0)

	/* Program Misc Delay Control values */
	li	a1, MEMCTRL_MISCDLYCTL_VAL
	sw	a1, MEMCTRL_MISCDLYCTL(a0)

	/* Program SDRAM refresh values */
	li	a1, MEMCTRL_REFRESH_VAL
	sw	a1, MEMCTRL_REFRESH(a0)

	/* Issue SDRAM init sequence */

	/* Enable memory controller */
	li	a1, MEMCTRL_CONTROL_SEQEN
	sw	a1, MEMCTRL_CONTROL(a0)

	/* Issue PRECHARGE command */
	li	a1, MEMCTRL_CONTROL_PRECMD
	sw	a1, MEMCTRL_CONTROL(a0)

	/* Issue two (2) AUTO REFRESH commands */
	li	a1, MEMCTRL_CONTROL_REFCMD
	sw	a1, MEMCTRL_CONTROL(a0)
	li	a1, MEMCTRL_CONTROL_REFCMD
	sw	a1, MEMCTRL_CONTROL(a0)

	/* Load mode register */
	li	a1, MEMCTRL_MODEREG_VAL
	sw	a1, MEMCTRL_MODEBUF(a0)
	li	a1, MEMCTRL_CONTROL_MRSCMD
	sw	a1, MEMCTRL_CONTROL(a0)

	/* Enable SDRAM access */
	li	a1, MEMCTRL_CONTROL_ACCEN
	sw	a1, MEMCTRL_CONTROL(a0)

	j	ra
	nop


/* In order to use same u-boot image for both 64MB and 128MB
 * using different memsetup functions that set the different
 * colsize in the memctrl config register
 */
	.globl	memsetup64
memsetup64:

	/* Memory controller core reset */
	li	a0, 0xB8008f98
	li	a1, 0x00030001
	sw	a1, 0(a0)
	lw	a1, 0(a0)	/* use multiple loads to insert a delay */
	lw	a1, 0(a0)
	lw	a1, 0(a0)
	li	a1, 0x00030000
	sw	a1, 0(a0)
	lw	a1, 0(a0)	/* use multiple loads to insert a delay */
	lw	a1, 0(a0)
	lw	a1, 0(a0)
	li	a1, 0x00010000
	sw	a1, 0(a0)
	lw	a1, 0(a0)	/* use multiple loads to insert a delay */
	lw	a1, 0(a0)
	lw	a1, 0(a0)

	/* Memory controller init */
	li	a0, 0xB8008000	# SDRAM registers

	/* Program SDRAM configuration values */
	li	a1, MEMCTRL_CONFIG_VAL_64MB
	sw	a1, MEMCTRL_CONFIG(a0)

	/* Program Read NCDL Correction values */
	li	a1, MEMCTRL_RDNCDLCOR_VAL
	sw	a1, MEMCTRL_RDNCDLCOR(a0)

	/* Program Misc Delay Control values */
	li	a1, MEMCTRL_MISCDLYCTL_VAL
	sw	a1, MEMCTRL_MISCDLYCTL(a0)

	/* Program SDRAM refresh values */
	li	a1, MEMCTRL_REFRESH_VAL
	sw	a1, MEMCTRL_REFRESH(a0)

	/* Issue SDRAM init sequence */

	/* Enable memory controller */
	li	a1, MEMCTRL_CONTROL_SEQEN
	sw	a1, MEMCTRL_CONTROL(a0)

	/* Issue PRECHARGE command */
	li	a1, MEMCTRL_CONTROL_PRECMD
	sw	a1, MEMCTRL_CONTROL(a0)

	/* Issue two (2) AUTO REFRESH commands */
	li	a1, MEMCTRL_CONTROL_REFCMD
	sw	a1, MEMCTRL_CONTROL(a0)
	li	a1, MEMCTRL_CONTROL_REFCMD
	sw	a1, MEMCTRL_CONTROL(a0)

	/* Load mode register */
	li	a1, MEMCTRL_MODEREG_VAL
	sw	a1, MEMCTRL_MODEBUF(a0)
	li	a1, MEMCTRL_CONTROL_MRSCMD
	sw	a1, MEMCTRL_CONTROL(a0)

	/* Enable SDRAM access */
	li	a1, MEMCTRL_CONTROL_ACCEN
	sw	a1, MEMCTRL_CONTROL(a0)

	j	ra
	nop
